/***************************************************************************
 * Artekit_DS1340.c
 * Copyright (C) 2014 Artekit Italy
 * http://www.artekit.eu
 *
 * This class provides support for the AK-DS1340, RTC with Trickle Charger,
 * to be used with Arduino.
 * This class depends on the Wire.h library, that must be included in
 * the main sketch.
 *
 * Product page:
 * http://www.artekit.eu/products/breakout-boards/ak-ds1340/
 *
***************************************************************************/

#include <Wire.h>
#include "Artekit_DS1340.h"

#define DS1340_ADDR 0x68

/* Strings for the days of the week */
static const char* days_of_week[] =
{
  "Monday",
  "Tuesday",
  "Wednesday",
  "Thursday",
  "Friday",
  "Saturday",
  "Sunday"
};

/* Converts a value from decimal to BCD */
static byte DECtoBCD(byte val)
{
  return ( (val/10*16) + (val%10) );
}

/* Converts a value from BCD to decimal */
static byte BCDtoDEC(byte val)
{
  return ( (val/16*10) + (val%16) );
}

Artekit_DS1340::Artekit_DS1340()
{
}

Artekit_DS1340::~Artekit_DS1340()
{
}

void Artekit_DS1340::GetTime(DATETIME* dt)
{
    Wire.beginTransmission(DS1340_ADDR);
    Wire.write(0);
    Wire.endTransmission();

    Wire.requestFrom(DS1340_ADDR, 7);
    dt->second = BCDtoDEC(Wire.read() & 0x7F);
    dt->minute = BCDtoDEC(Wire.read());
    dt->hour = BCDtoDEC(Wire.read());
    dt->day_of_week = BCDtoDEC(Wire.read());
    dt->day = BCDtoDEC(Wire.read());
    dt->month = BCDtoDEC(Wire.read());
    dt->year = BCDtoDEC(Wire.read()) + 2000;
}

void Artekit_DS1340::SetTime(DATETIME* dt)
{
    Wire.beginTransmission(DS1340_ADDR);
    Wire.write(0);
    Wire.write(DECtoBCD(dt->second));
    Wire.write(DECtoBCD(dt->minute));
    Wire.write(DECtoBCD(dt->hour));
    Wire.write(DECtoBCD(dt->day_of_week));
    Wire.write(DECtoBCD(dt->day));
    Wire.write(DECtoBCD(dt->month));
    Wire.write(DECtoBCD(dt->year - 2000));
    Wire.write(0);
    Wire.endTransmission();
}

const char* Artekit_DS1340::GetDayOfWeek(uint8_t day_of_week)
{
    if (!day_of_week || day_of_week > 7) return NULL;
    return days_of_week[day_of_week-1];
}

void Artekit_DS1340::EnableTrickleCharger()
{
    Wire.beginTransmission(DS1340_ADDR);
    Wire.write(0x08);

    /* One diode, 4k resistor*/	
    Wire.write(0xAB);
    Wire.endTransmission();
}

void Artekit_DS1340::DisableTrickleCharger()
{
    Wire.beginTransmission(DS1340_ADDR);
    Wire.write(0x08);	
    Wire.write(0x00);
    Wire.endTransmission();
}

uint8_t Artekit_DS1340::ReadControlReg()
{
    Wire.beginTransmission(DS1340_ADDR);
    Wire.write(0x07);	
    Wire.endTransmission();

    Wire.requestFrom(DS1340_ADDR, 1);
    return Wire.read();
}

void Artekit_DS1340::WriteControlReg(uint8_t value)
{
    Wire.beginTransmission(DS1340_ADDR);
    Wire.write(0x07);	
    Wire.write(value);
    Wire.endTransmission();
}

void Artekit_DS1340::EnableFTOUT()
{
    uint8_t reg = ReadControlReg();
    reg |= 0xC0;
    WriteControlReg(reg);
}

void Artekit_DS1340::DisableFTOUT()
{
    uint8_t reg = ReadControlReg();
    reg &= ~0xC0;
    WriteControlReg(reg);
}
